check_recipe <- function(recipe, logger) {

  # recipe: yaml recipe already read it
  # output: errors or the total number of workflow (vars x regions) to compute

  info(logger, paste("Checking recipe", recipe$filename))

  # ---------------------------------------------------------------------
  # ANALYSIS CHECKS
  # ---------------------------------------------------------------------

  TIME_SETTINGS = c('sdate','leadtimemin','leadtimemax','hcst_start','hcst_end')
  PARAMS = c('Horizon','Time','Variables','Region','Regrid','Workflow','Datasets')
  HORIZONS <- c('Subseasonal','Seasonal','Decadal')

  # create output dirs:
  if (!any(names(recipe) %in% "Analysis")) {
    error(logger, "The recipe should contain an element called 'Analysis'.")
  }

  if (!all(PARAMS %in% names(recipe$Analysis))) {
    error(logger, 
          paste("The element 'Analysis' in the recipe should contain these",
         "elements:", paste(PARAMS, collapse = " ")))
  }

  if (!any(HORIZONS %in% recipe$Analysis$Horizon)) {
    error(logger,
         "The element 'Horizon' in the recipe should be one of the followings:",
         paste(HORIZONS, collapse = " "))
  }
  # Check temporal settings and
  # count the number of verifications
  if (!all(TIME_SETTINGS %in% names(recipe$Analysis$Time))) {
    error(logger, 
       paste("The element 'Time' in the recipe should contain these elements:",
             paste(TIME_SETTINGS, collapse = " ")))
  }
  if (is.null(recipe$Analysis$Time$sdate$fcst_year) ||
    recipe$Analysis$Time$sdate$fcst_year == 'None') {
    stream <- "hindcast"
    recipe$Analysis$Time$sdate$fcst_year <- 'YYYY'
  } else {
    stream <- "fcst"
  }
  if (length(recipe$Analysis$Time$sdate$fcst_day) > 1 &&
      tolower(recipe$Analysis$Horizon) != "subseasonal") {
    warn(logger,
         paste("Only subseasonal verification allows multiple forecast days."),
               "Element fcst_day in recipe set as 1.")
    recipe$Analysis$Time$sdate$fcst_day <- '01'
  }
  if (is.null(recipe$Analysis$Time$sdate$fcst_month)) {
    error(logger,
          paste("The element 'fcst_month' in the recipe should be defined."))
  }

  
  fcst.sdate <- NULL
  for (year in recipe$Analysis$Time$sdate$fcst_year) {
    for (month in recipe$Analysis$Time$sdate$fcst_month) {
      for (day in recipe$Analysis$Time$sdate$fcst_day) {
        fcst.sdate <- c(fcst.sdate, 
                             paste0(year,
                               sprintf("%02d", as.numeric(month)),
                               sprintf("%02d", as.numeric(day))))
      }
    }
  }
  fcst.sdate <- list(stream = stream, fcst.sdate = fcst.sdate)   
  # Regrid checks:
  if (length(recipe$Analysis$Regrid) != 2) {
    error(logger,
          "The 'Regrid' element should specified the 'method' and 'type'.")
  }
# more checks
  # ...
  # calculate number of workflows to create for each variable and 
  if (length(recipe$Analysis$Horizon) > 1) {
     error(logger, "Only 1 Horizon can be specified in the recipe")
  }
  nvar <- length(recipe$Analysis$Variables)
  if (nvar > 2) {
    error(logger,
          "Only two type of Variables can be listed: ECVs and Indicators.")
  }
  # remove NULL or None Indicators or ECVs from the recipe:
  if (!is.list(recipe$Analysis$Variables$Indicators)) {
    recipe$Analysis$Variables <- recipe$Analysis$Variables[
           -which(names(recipe$Analysis$Variables) == 'Indicators')]
  }
  if (!is.list(recipe$Analysis$Variables$ECVs)) {
    recipe$Analysis$Variables <- recipe$Analysis$Variables[
           -which(names(recipe$Analysis$Variables) == 'ECVs')]
  }
  if (length(recipe$Analysis$Region) > 3) {
    error(logger, "The section 'Region' in the recipe should be checked.",
         "Global, Countries and Regional elements are expected.")
  }
  if (!is.logical(recipe$Analysis$Region$Global)) {
    error(logger, "The element 'Global' in Region should be logical.")
  }
  nregions <- 0
  if (recipe$Analysis$Region$Global == TRUE) {
    nregions <- 1
  }
  if (recipe$Analysis$Region$Aggregation == TRUE) {
    nregions <- nregions + 1
  }
  if ("Regional" %in% names(recipe$Analysis$Region)) {
    nregions <- nregions + length(recipe$Analysis$Region$Regional)
    limits <- c('latmin', 'latmax', 'lonmin', 'lonmax')
    for (i in 1:length(recipe$Analysis$Region$Regional)) {
      if (!all(limits %in% names(recipe$Analysis$Region$Regional[[i]]))) {
        error(logger,
           "Each region defined in element 'Regional' should have 4 elements:",
              paste(limits, collapse = " "))
      }
      # are numeric? class list mode list 
    }
  }
  
  # ---------------------------------------------------------------------
  # RUN CHECKS
  # ---------------------------------------------------------------------

  RUN_FIELDS = c("Loglevel","Terminal","output_dir","code_dir")
  LOG_LEVELS = c("INFO","DEBUG","WARNING","ERROR")

  if (!any(names(recipe) %in% "Run")) {
    error(logger, "The recipe should contain an element called 'Run'.")
  }
  if (!all(RUN_FIELDS %in% names(recipe$Run))) {
    error(logger, paste0("Run should contain the fields: ", 
                         paste(RUN_FIELDS,collapse=", "), "."))
  }
  if (!is.character(recipe$Run$output_dir)) {
    error(logger,
          paste("The Run element 'output_dir' in", recipe$filename,"file ",
                "should be a character string indicating the path ",
                "where to save the outputs."))
  }
  if (!is.character(recipe$Run$code_dir)) {
    error(logger,
          paste("The Run element 'code_dir' in", recipe$filename,"file ",
                "should be a character string indicating the path ",
                "where the code is."))
  }
  if (!is.logical(recipe$Run$Terminal)) {
    error(logger,
          paste("The Run element 'Terminal' in", recipe$filename,"file ",
                "should be a boolean value indicating wether to print or not the log",
                "in the terminal."))
  }
  if (!is.character(recipe$Run$Loglevel) || !any(recipe$Run$Loglevel %in% LOG_LEVELS)) {
    error(logger,
          paste("The Run element 'Loglevel' in", recipe$filename,"file ",
                "should be a character string indicating one of the levels available: ",
                paste0(LOG_LEVELS,collapse='/')))
  }

  # ---------------------------------------------------------------------
  # WORKFLOW CHECKS
  # ---------------------------------------------------------------------
  # Check workflow: need to define restrictions? 
  # e.g. only one calibration method
  nverifications <- check_number_of_dependent_verifications(recipe)
  info(logger, paste("Start Dates", paste(fcst.sdate, collapse = " ")))
  info(logger, "Recipe checked succsessfully.")
  return(append(nverifications, fcst.sdate))
}

check_number_of_dependent_verifications <- function(recipe) {
  # Number of verifications depends on the variables and indicators requested
  # and the order of the workflow:
  # workflow: correction + indicator --> only 1 variable is calibrated
  # workflow: indicator + correction --> the indicator and the ecv are calibrated
  independent_verifications <- NULL
  dependent_verifications <- NULL
  dep <- 1
  # check workflow order:
  if (all(c('Calibration', 'Indicators') %in% names(recipe$Analysis$Workflow))) {
    cal_pos <- which(names(recipe$Analysis$Workflow) == 'Calibration')
    ind_pos <- which(names(recipe$Analysis$Workflow) == 'Indicators')
    if (cal_pos < ind_pos) {
      workflow_independent <- FALSE
    } else {
      workflow_independent <- TRUE
    }
  }
  if (workflow_independent) {
    independent_verifications <- append(recipe$Analysis$Variables$ECVs,
                                        recipe$Analysis$Variables$Indicators)
  } else { 
    if (is.null(recipe$Analysis$Variables$Indicators) || 
       (length(recipe$Analysis$Variables$Indicators) == 1 && 
        is.null(recipe$Analysis$Variables$ECVs))) {
      independent_verifications <- append(recipe$Analysis$Variables$ECVs,
                                          recipe$Analysis$Variables$Indicators)
    } else { 
      ecvs <- recipe$Analysi$Variables$ECVs
      inds <- recipe$Analysi$Variables$Indicators
      ind_table <- read_yaml(paste0(recipe$Run$code_dir,
                             "conf/indicators_table.yml"))
      # first, loop on ecvs if any and compare to indicators
      done <- NULL # to gather the indicators reviewed
      if (!is.null(ecvs)) {
        for (i in 1:length(ecvs)) {
          dependent <- list(ecvs[[i]])
          for (j in 1:length(inds)) {
            if (ind_table[inds[[j]]$name][[1]]$ECVs == ecvs[[i]]$name) {
              if (ind_table[inds[[j]]$name][[1]]$freq == ecvs[[i]]$freq) {
              # they are dependent
                dependent <- append(dependent, inds[[j]])
                done <- append(done, inds[[j]])
              }
            }
          }
          if (length(dependent) == 1) {
            dependent <- NULL
            independent_verifications <- append(independent_verifications,
                                              list(ecvs[[i]]))
          } else {
            dependent_verifications <- append(dependent_verifications,
                                              list(dependent))
          }  
        }
        # There are indicators not reviewed yet?
        if (length(done) < length(inds)) {
          if (length(inds) == 1) {
            independent_verifications <- append(independent_verifications,
                                                inds)
          } else {
            done <- NULL
            for (i in 1:(length(inds) - 1)) {
              dependent <- list(inds[[i]]$name)
              if (is.na(match(unlist(dependent), unlist(done)))) {
                for (j in (i+1):length(inds)) {
                  if (ind_table[inds[[i]]$name][[1]]$ECVs == 
                      ind_table[inds[[j]]$name][[1]]$ECVs) {
                    if (ind_table[inds[[i]]$name][[1]]$freq ==
                        ind_table[inds[[j]]$name][[1]]$freq) {
                      dependent <- append(dependent, inds[[j]]$name)
                      done <- dependent
                    }
                  }
                }
              } 
              if (length(dependent) == 1) {
                independent_verifications <- dependent
                dependent <- NULL     
              } else {
                dependent_verifications <- dependent
              }
            } 
          }
        }
      } else { # there are only Indicators:
        done <- NULL
        for (i in 1:(length(inds) - 1)) {
          dependent <- list(inds[[i]]$name)
          if (is.na(match(unlist(dependent), unlist(done)))) {
            for (j in (i+1):length(inds)) {
              if (ind_table[inds[[i]]$name][[1]]$ECVs ==
                  ind_table[inds[[j]]$name][[1]]$ECVs) {
                if (ind_table[inds[[i]]$name][[1]]$freq ==
                    ind_table[inds[[j]]$name][[1]]$freq) {
                  dependent <- append(dependent, inds[[j]]$name)
                  done <- dependent
                }
              }
            }
          }
          if (length(dependent) == 1) {
            independent_verifications <- dependent
            dependent <- NULL
          } else {
            dependent_verifications <- dependent
          }
        }
      } 
    }
  }
  if (!is.null(independent_verifications)) {   
    info(logger, paste("The variables for independent verification are ", 
         paste(independent_verifications, collapse = " ")))
  }
  if (!is.null(dependent_verifications)) {
    info(logger, paste("The variables for dependent verification are: ",
         paste(dependent_verifications, collapse = " ")))
  }
  # remove unnecessary names in objects to be removed
  return(list(independent = independent_verifications, 
              dependent = dependent_verifications))
}
#workflow <- list(Calibration = list(method = 'SBC'),
#                 Skill = list(metric = 'RPSS'))
#ApplyWorkflow <- function(workflow) {

#res <- do.call('CST_BiasCorrection',
#               args = list(exp = lonlat_data$exp,
#                           obs = lonlat_data$obs))
   



